/*
 GNU Lesser General Public License

 FormatAction
 Copyright (C) 2000 Howard Kistler

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.hexidec.ekit.action;

import java.awt.event.ActionEvent;
import javax.swing.JTextPane;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.html.HTML;

import com.hexidec.ekit.EkitCore;
import com.hexidec.ekit.component.*;

import com.hexidec.util.Translatrix;
import java.util.Enumeration;
import javax.swing.text.StyleConstants;
import javax.swing.JEditorPane;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.AttributeSet;

/** Class for implementing HTML format actions
 * (NOTE : Does not toggle. User must use the "Clear Format" option to remove formatting correctly.)
 * 050714 Jakob sikken, it does toggle now and it also works when nothing is selected
 * The code is copied from StyledEditorKit.BoldAction,
 * after I "dicovered" that StyleConstants has function for strike through, sub and superscript,
 * just as for bold.
 */
public class FormatAction extends StyledEditorKit.StyledTextAction
{
	protected EkitCore parentEkit;
	HTML.Tag htmlTag;

	public FormatAction(EkitCore ekit, String actionName, HTML.Tag inTag)
	{
		super(actionName);
		parentEkit = ekit;
		htmlTag = inTag;
	}

	public void actionPerformed(ActionEvent ae)
	{
		// allow it also when no text is selected
		/*
		JTextPane parentTextPane = parentEkit.getTextPane();
		String selText = parentTextPane.getSelectedText();
		int textLength = -1;
		if (selText != null)
		{
			textLength = selText.length();
		}
		if (selText == null || textLength < 1)
		{
			SimpleInfoDialog sidWarn = new SimpleInfoDialog(parentEkit.getFrame(), "", true,
				Translatrix.getTranslationString("ErrorNoTextSelected"), SimpleInfoDialog.ERROR);
		}
		else
	   */
		{
			realAactionPerformed(ae);
		}
	}

	public void realAactionPerformed(ActionEvent e)
	{
		JEditorPane editor = getEditor(e);
		if (editor != null)
		{
			StyledEditorKit kit = getStyledEditorKit(editor);
			MutableAttributeSet attr = kit.getInputAttributes();
			boolean set = !isTagSet(attr);
			SimpleAttributeSet sas = new SimpleAttributeSet();
			setTag(sas, set);
			setCharacterAttributes(editor, sas, false);
		}
	}

	/**
	 * Checks whether the bold attribute is set.
	 *
	 * @param a the attribute set
	 * @return true if set else false
	 */
	public boolean isTagSet(AttributeSet a)
	{
		if (htmlTag.equals(HTML.Tag.STRIKE))
			return StyleConstants.isStrikeThrough(a);
		else if (htmlTag.equals(HTML.Tag.SUP))
			return StyleConstants.isSuperscript(a);
		else if (htmlTag.equals(HTML.Tag.SUB))
			return StyleConstants.isSubscript(a);
		else
		{
			System.err.println("FormatAction: Unsupported HTML.Tag " + htmlTag.toString());
			return false;
		}
	}

	/**
	 * Sets the bold attribute.
	 *
	 * @param a the attribute set
	 * @param b specifies true/false for setting the attribute
	 */
	public void setTag(MutableAttributeSet a, boolean b)
	{
		if (htmlTag.equals(HTML.Tag.STRIKE))
			StyleConstants.setStrikeThrough(a,b);
		else if (htmlTag.equals(HTML.Tag.SUP))
			StyleConstants.setSuperscript(a,b);
		else if (htmlTag.equals(HTML.Tag.SUB))
			StyleConstants.setSubscript(a,b);
		else
		{
			System.err.println("FormatAction: Unsupported HTML.Tag " + htmlTag.toString());
		}
	}

	public void dumpAttributeSet(AttributeSet set, String label)
	{
		System.out.print("SimpleAttributeSet (" + label + "): ");
		Enumeration enumeration = set.getAttributeNames();
		while (enumeration.hasMoreElements())
		{
			Object key = enumeration.nextElement();
			Object value = set.getAttribute(key);
			System.out.print(key + "=" + value + ",");
		}
		System.out.println();
	}
}
